%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Kristian L. Jenssen & Kaan H. Yabar
%   Engineering Cybernetics - April.2011 
%
%   LIVE PLOT OF THE ORIENTATION OF THE PROTOTYPE (GRAPH)
%   Requires that a microcontroller running both EQUST and EKF is
%   connected to the computer serial port 3 with baudrate 115200.
%   The microcontroller must send estimated quaternions in following form:
%   'q1*10000,q2*10000,q3*10000,q4*10000,1' <-EQUEST
%   'q1*10000,q2*10000,q3*10000,q4*10000,2' <-EKF
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(1)

%Serial port open
s = serial('COM3');
s.BaudRate = 115200;
fopen(s);
pause(1);

go = true;
count=1;

dat = '0,0,0,1000';
t=0;
x_f = 0;
y_f = 0;
z_f = 0;
xx_f = 0;
yy_f = 0;
zz_f = 0;
col = 'red';
Ts = 0.14;

%Stop after 70 sec
while(go<40)
    %Read data from microcontroller
    dat=fscanf(s);
    [qua]=strread(dat,'%d','delimiter', ',','whitespace','\n');
    if(count > 3)
            
            qua(1:4)=qua(1:4)/10000;
            euler_ang = qua_to_euler(qua(1:4));
            
            xx=euler_ang(1);
            yy=euler_ang(2);
            zz=euler_ang(3);
            
            %Plot graph
            subplot(3,1,1)
            hold on;
            if(qua(5) == 1)
                col = 'red';
                plot([t-Ts t],[x_f xx]*180/pi,'LineWidth',1,'color',col);
            else
               col = 'blue';
               plot([t-Ts t],[xx_f xx]*180/pi,'LineWidth',1,'color',col); 
            end
            axis([0 inf -180 180]); 
            legend('EQUEST','EKF')
            xlabel('Time [s]');
            ylabel('Roll [Deg]');
            
            subplot(3,1,2)
            hold on;
            if(qua(5) == 1)
                col = 'red';
                plot([t-Ts t],[y_f yy]*180/pi,'LineWidth',1,'color',col);
            else
               col = 'blue';
               plot([t-Ts t],[yy_f yy]*180/pi,'LineWidth',1,'color',col); 
            end
            axis([0 inf -180 180]); 
            xlabel('Time [s]');
            ylabel('Pitch [Deg]');
            
            subplot(3,1,3)
            hold on;
            if(qua(5) == 1)
                col = 'red';
                plot([t-Ts t],[z_f zz]*180/pi,'LineWidth',1,'color',col);
            else
               col = 'blue';
               plot([t-Ts t],[zz_f zz]*180/pi,'LineWidth',1,'color',col); 
            end
            axis([0 inf -180 180]); 
            xlabel('Time [s]');
            ylabel('Yaw [Deg]');
            
            if(qua(5) == 1)
                x_f = xx;
                y_f = yy;
                z_f = zz;
                
            else
                xx_f = xx;
                yy_f = yy;
                zz_f = zz;
                
            end    

            drawnow();
            hold off;
            t=t+Ts;
            count = 1;
            go = t;
    end
    count = count + 1;

end
fclose(s);



