%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Kristian L. Jenssen & Kaan H. Yabar
%   Engineering Cybernetics - April.2011 
%
%   LIVE PLOT OF ORIENTATION (3D CUBE)
%   Requires that a microcontroller running either EQUST or EKF is
%   connected to the computer serial port 3 with baudrate 115200.
%   The microcontroller must send estimated quaternions in following form:
%   'q1*10000,q2*10000,q3*10000,q4*10000'
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(1)

%Serial port open
s = serial('COM3');
s.BaudRate = 115200;
fopen(s);
pause(1);

go = true;
count=1;

%Cube sides (Cube is centered to the (0,0,0) coordinates)
side1o=[0 0.4 0.4 0;0 0 0 0;0 0 0.2 0.2]-[0.2 0.2 0.2 0.2;0.5 0.5 0.5 0.5;0.1 0.1 0.1 0.1];
side2o=[0 0 0 0;0 1 1 0;0 0 0.2 0.2]-[0.2 0.2 0.2 0.2;0.5 0.5 0.5 0.5;0.1 0.1 0.1 0.1];
side3o=[0 0.4 0.4 0;0 0 1 1;0 0 0 0]-[0.2 0.2 0.2 0.2;0.5 0.5 0.5 0.5;0.1 0.1 0.1 0.1];
side4o=[0 0.4 0.4 0;1 1 1 1;0 0 0.2 0.2]-[0.2 0.2 0.2 0.2;0.5 0.5 0.5 0.5;0.1 0.1 0.1 0.1];
side5o=[0.4 0.4 0.4 0.4;0 1 1 0;0 0 0.2 0.2]-[0.2 0.2 0.2 0.2;0.5 0.5 0.5 0.5;0.1 0.1 0.1 0.1];
side6o=[0 0.4 0.4 0;0 0 1 1;0.2 0.2 0.2 0.2]-[0.2 0.2 0.2 0.2;0.5 0.5 0.5 0.5;0.1 0.1 0.1 0.1];

dat = '0,0,0,1000';

%Color cube
G1=[0,0,1];
G2=[0,1,0];
G3=[1,0,0];
G4=[0,0.5,0.5];
G5=[0.5,0.5,0];
G6=[0.5,0,0.5];

%Rotating the cube to its initial position 
Rot90z=[cos(pi/6) -sin(pi/6) 0;sin(pi/6) cos(pi/6) 0;0 0 1];
Rot90y=[1 0 0;0 cos(-pi/2) -sin(-pi/2);0 sin(-pi/2) cos(-pi/2)];

while(go)
    %Read data from microcontroller
    dat=fscanf(s);
    [qua]=strread(dat,'%d','delimiter', ',','whitespace','\n');
    if(count > 1)
            qua=qua/10000;
            euler_ang = qua_to_euler(qua);
            
            %Draw cube
            clf
            
            xx=euler_ang(1);
            yy=euler_ang(2);
            zz=euler_ang(3);
            RR=[cos(zz)*cos(yy) -sin(zz)*cos(xx)+cos(zz)*sin(yy)*sin(xx) sin(zz)*sin(xx)+cos(zz)*cos(xx)*sin(yy);
            sin(zz)*cos(yy) cos(zz)*cos(xx)+sin(xx)*sin(yy)*sin(zz) -cos(zz)*sin(xx)+sin(yy)*sin(zz)*cos(xx);
            -sin(yy) cos(yy)*sin(xx) cos(yy)*cos(xx)];
        
            %Rotation of cube
            side1=Rot90y*Rot90z*RR*side1o;
            side2=Rot90y*Rot90z*RR*side2o;
            side3=Rot90y*Rot90z*RR*side3o;
            side4=Rot90y*Rot90z*RR*side4o;
            side5=Rot90y*Rot90z*RR*side5o;
            side6=Rot90y*Rot90z*RR*side6o;
            
            hold on;
            
            %Coloring the cube
            fill3(side1(1,:),side1(2,:),side1(3,:),G1,'FaceAlpha',0.8);
            fill3(side2(1,:),side2(2,:),side2(3,:),G2,'FaceAlpha',0.8);
            fill3(side3(1,:),side3(2,:),side3(3,:),G3,'FaceAlpha',0.8);
            fill3(side4(1,:),side4(2,:),side4(3,:),G4,'FaceAlpha',0.8);
            fill3(side5(1,:),side5(2,:),side5(3,:),G5,'FaceAlpha',0.8);
            fill3(side6(1,:),side6(2,:),side6(3,:),G6,'FaceAlpha',0.8);
 
            
            axis([-1.5 1.5 -1.5 1.5 -1.5 1.5])
            drawnow();
            hold off;

            count = 1;
    end
    count = count + 1;

end
fclose(s);

