%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Kristian L. Jenssen & Kaan H. Yabar
%   Engineering Cybernetics - April.2011 
%
%   LIVE PLOT OF THE ORIENTATION OF THE SPARKFUN IMU (3D CUBE PLOT)
%   Requires that the 9-DOF sparkfun IMU is connected to the computer 
%   on serial port 3 (The input port can be changed in code).
%
%   NOTE: For best view, rotate the pc-screen until it is in line with the
%   orientation of the sensor.
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(1)

%Local magnetic field for Trondheim
magnometer=[13598.5;444.7;-49854.8];
magnometer=magnometer/norm(magnometer);

Ts = 0.0011;    %Time step
x = zeros(10,1);
x(1:4,1) = [0 0 0 1]';

%Covariance matrix for process and sensors
r1 = 0.05;
r2 = 0.08;
R = diag([r1,r1,r1,r2,r2,r2]);

P = 0*eye(10);

%Normalized gravity vector
a=[0 0 1];
a1=a(1);
a2=a(2);
a3=a(3);

%Serial port open
s = serial('COM3');
s.BaudRate = 38400;
fopen(s);
pause(5);

%Writing '4' to the Sparkfun IMU gives raw data output
fwrite(s, '4','char');

go = true;
count=1;

%Cube sides (Cube is centered to the (0,0,0) coordinates)
side1o=[0 1 1 0;0 0 0 0;0 0 1 1]-[0.5 0.5 0.5 0.5;0.5 0.5 0.5 0.5;0.5 0.5 0.5 0.5];
side2o=[0 0 0 0;0 1 1 0;0 0 1 1]-[0.5 0.5 0.5 0.5;0.5 0.5 0.5 0.5;0.5 0.5 0.5 0.5];
side3o=[0 1 1 0;0 0 1 1;0 0 0 0]-[0.5 0.5 0.5 0.5;0.5 0.5 0.5 0.5;0.5 0.5 0.5 0.5];
side4o=[0 1 1 0;1 1 1 1;0 0 1 1]-[0.5 0.5 0.5 0.5;0.5 0.5 0.5 0.5;0.5 0.5 0.5 0.5];
side5o=[1 1 1 1;0 1 1 0;0 0 1 1]-[0.5 0.5 0.5 0.5;0.5 0.5 0.5 0.5;0.5 0.5 0.5 0.5];
side6o=[0 1 1 0;0 0 1 1;1 1 1 1]-[0.5 0.5 0.5 0.5;0.5 0.5 0.5 0.5;0.5 0.5 0.5 0.5];

%Color cube
G1=[0,0,1];
G2=[0,1,0];
G3=[1,0,0];
G4=[0,0.5,0.5];
G5=[0.5,0.5,0];
G6=[0.5,0,0.5];

%Rotating the cube to its initial position 
Rot90z=[cos(pi/6) -sin(pi/6) 0;sin(pi/6) cos(pi/6) 0;0 0 1];
Rot90y=[1 0 0;0 cos(-pi/2) -sin(-pi/2);0 sin(-pi/2) cos(-pi/2)];

while(go)
    dat=fscanf(s);
    if(dat(2)=='$')
            %Reading data from IMU
            str=dat(4:length(dat)-4);
            [data_tot]=strread(str,'%f','delimiter', ',');
            
            %Saving magnetometer, accelerometer and gyro data
            acc = data_tot(1:3);
            omega_real = data_tot(4:6)-[260 257 260]';
            mag = data_tot(7:9);
            omega_real = omega_real*0.92*(pi/180);
            mag = mag/norm(mag);
            acc = acc/norm(acc);
            z = [acc;mag];
            
            %EKF
            omega_skew=[0 -omega_real(3) omega_real(2);omega_real(3) 0 -omega_real(1);-omega_real(2) omega_real(1) 0];
            Omega = 0.5 * [omega_skew omega_real; -omega_real' 0];
    
            m1 = magnometer(1);
            m2 = magnometer(2);
            m3 = magnometer(3);
         
            A = [expm(Omega*Ts) zeros(4,3) zeros(4,3);
            zeros(3,4) eye(3) zeros(3,3);
            zeros(3,4) zeros(3,3) eye(3)];
        
            %Extended Kalman Filter Equations
            
            %Time update equations
            x = A*x;
            x(1:4)=x(1:4)/norm(x(1:4));
    
            q1 = x(1);
            q2 = x(2);
            q3 = x(3);
            q4 = x(4);
            q_13=[0 -q3 q2;q3 0 -q1;-q2 q1 0];
            E=[q4*eye(3)+q_13;-q1 -q2 -q3];
            Y=[q4*eye(3)-q_13;-q1 -q2 -q3];
            Rot=E'*Y;
            
            %Linearization of measurment matrix
            num = (q1^2+q2^2+q3^2+q4^2)^(3/2);

            daxdq1 = a1*q1^3+3*a1*q1*q2^2+3*a1*q1*q3^2+a1*q1*q4^2-2*a2*q1*q3*q4+2*a2*q2^3+2*a2*q2*q3^2+2*a2*q2*q4^2+2*a3*q1*q2*q4+2*a3*q2^2*q3+2*a3*q3^3+2*a3*q3*q4^2;
            daxdq2 = -3*a1*q1^2*q2-a1*q2^3-a1*q2*q3^2-3*a1*q2*q4^2+2*a2*q1^3+2*a2*q1*q3^2+2*a2*q1*q4^2-2*a2*q2*q3*q4-2*a3*q1^2*q4-2*a3*q1*q2*q3-2*a3*q3^2*q4-2*a3*q4^3;
            daxdq3 = -3*a1*q1^2*q3-a1*q2^2*q3-a1*q3^3-3*a1*q3*q4^2+2*a2*q1^2*q4-2*a2*q1*q2*q3+2*a2*q2^2*q4+2*a2*q4^3+2*a3*q1^3+2*a3*q1*q2^2+2*a3*q1*q4^2+2*a3*q2*q3*q4;
            daxdq4 = -(-a1*q1^2*q4-3*a1*q2^2*q4-3*a1*q3^2*q4-a1*q4^3-2*a2*q1^2*q3+2*a2*q1*q2*q4-2*a2*q2^2*q3-2*a2*q3^3+2*a3*q1^2*q2+2*a3*q1*q3*q4+2*a3*q2^3+2*a3*q2*q3^2);
            daydq1 = 2*(a1*(q1*q3*q4+q2^3+q2*(q3^2+q4^2))+a3*(q4*(q2^2+q3^2+q4^2)-q1*q2*q3))-a2*q1*(q1^2+3*q2^2+q3^2+3*q4^2);
            daydq2 = 2*a1*q1^3+2*a1*q1*q3^2+2*a1*q1*q4^2+2*a1*q2*q3*q4+3*a2*q1^2*q2+a2*q2^3+3*a2*q2*q3^2+a2*q2*q4^2+2*a3*q1^2*q3-2*a3*q1*q2*q4+2*a3*q3^3+2*a3*q3*q4^2;
            daydq3 = -2*a1*q1^2*q4-2*a1*q1*q2*q3-2*a1*q2^2*q4-2*a1*q4^3-a2*q1^2*q3-3*a2*q2^2*q3-a2*q3^3-3*a2*q3*q4^2+2*a3*q1^2*q2-2*a3*q1*q3*q4+2*a3*q2^3+2*a3*q2*q4^2;
            daydq4 = -2*a1*q1^2*q3-2*a1*q1*q2*q4-2*a1*q2^2*q3-2*a1*q3^3+3*a2*q1^2*q4+2*a2*q2^2*q4+3*a2*q3^2*q4+a2*q4^3+2*a3*q1^3+2*a3*q1*q2^2+2*a3*q1*q3^2-2*a3*q2*q3*q4;
            dazdq1 = -(2*a1*q1*q2*q4-2*a1*q2^2*q3-2*a1*q3^3-2*a1*q3*q4^2+2*a2*q1*q2*q3+2*a2*q2^2*q4+2*a2*q3^2*q4+2*a2*q4^3+a3*q1^3+a3*q1*q2^2+3*a3*q1*q3^2+3*a3*q1*q4^2);
            dazdq2 = 2*(a1*(q1^2*q4-q1*q2*q3+q3^2*q4+q4^3)+a2*(q1^2*q3+q1*q2*q4+q3^3+q3*q4^2))-a3*q2*(q1^2+q2^2+3*(q3^2+q4^2));
            dazdq3 = 2*a1*q1^3+2*a1*q1*q2^2+2*a1*q1*q4^2-2*a1*q2*q3*q4+2*a2*q1^2*q2+2*a2*q1*q3*q4+2*a2*q2^3+2*a2*q2*q4^2+3*a3*q1^2*q3+3*a3*q2^2*q3+a3*q3^3+a3*q3*q4^2;
            dazdq4 = -(-2*a1*q1^2*q2+2*a1*q1*q3*q4-2*a1*q2^3-2*a1*q2*q3^2+2*a2*q1^3+2*a2*q1*q2^2+2*a2*q1*q3^2+2*a2*q2*q3*q4-3*a3*q1^2*q4-3*a3*q2^2*q4-a3*q3^2*q4-a3*q4^3);
    
            dmxdq1 = m1*q1^3+3*m1*q1*q2^2+3*m1*q1*q3^2+m1*q1*q4^2-2*m2*q1*q3*q4+2*m2*q2^3+2*m2*q2*q3^2+2*m2*q2*q4^2+2*m3*q1*q2*q4+2*m3*q2^2*q3+2*m3*q3^3+2*m3*q3*q4^2;
            dmxdq2 = -3*m1*q1^2*q2-m1*q2^3-m1*q2*q3^2-3*m1*q2*q4^2+2*m2*q1^3+2*m2*q1*q3^2+2*m2*q1*q4^2-2*m2*q2*q3*q4-2*m3*q1^2*q4-2*m3*q1*q2*q3-2*m3*q3^2*q4-2*m3*q4^3;
            dmxdq3 = -3*m1*q1^2*q3-m1*q2^2*q3-m1*q3^3-3*m1*q3*q4^2+2*m2*q1^2*q4-2*m2*q1*q2*q3+2*m2*q2^2*q4+2*m2*q4^3+2*m3*q1^3+2*m3*q1*q2^2+2*m3*q1*q4^2+2*m3*q2*q3*q4;
            dmxdq4 = -(-m1*q1^2*q4-3*m1*q2^2*q4-3*m1*q3^2*q4-m1*q4^3-2*m2*q1^2*q3+2*m2*q1*q2*q4-2*m2*q2^2*q3-2*m2*q3^3+2*m3*q1^2*q2+2*m3*q1*q3*q4+2*m3*q2^3+2*m3*q2*q3^2);
            dmydq1 = 2*(m1*(q1*q3*q4+q2^3+q2*(q3^2+q4^2))+m3*(q4*(q2^2+q3^2+q4^2)-q1*q2*q3))-m2*q1*(q1^2+3*q2^2+q3^2+3*q4^2);
            dmydq2 = 2*m1*q1^3+2*m1*q1*q3^2+2*m1*q1*q4^2+2*m1*q2*q3*q4+3*m2*q1^2*q2+m2*q2^3+3*m2*q2*q3^2+m2*q2*q4^2+2*m3*q1^2*q3-2*m3*q1*q2*q4+2*m3*q3^3+2*m3*q3*q4^2;
            dmydq3 = -2*m1*q1^2*q4-2*m1*q1*q2*q3-2*m1*q2^2*q4-2*m1*q4^3-m2*q1^2*q3-3*m2*q2^2*q3-m2*q3^3-3*m2*q3*q4^2+2*m3*q1^2*q2-2*m3*q1*q3*q4+2*m3*q2^3+2*m3*q2*q4^2;
            dmydq4 = -2*m1*q1^2*q3-2*m1*q1*q2*q4-2*m1*q2^2*q3-2*m1*q3^3+3*m2*q1^2*q4+2*m2*q2^2*q4+3*m2*q3^2*q4+m2*q4^3+2*m3*q1^3+2*m3*q1*q2^2+2*m3*q1*q3^2-2*m3*q2*q3*q4;
            dmzdq1 = -(2*m1*q1*q2*q4-2*m1*q2^2*q3-2*m1*q3^3-2*m1*q3*q4^2+2*m2*q1*q2*q3+2*m2*q2^2*q4+2*m2*q3^2*q4+2*m2*q4^3+m3*q1^3+m3*q1*q2^2+3*m3*q1*q3^2+3*m3*q1*q4^2);
            dmzdq2 = 2*(m1*(q1^2*q4-q1*q2*q3+q3^2*q4+q4^3)+m2*(q1^2*q3+q1*q2*q4+q3^3+q3*q4^2))-m3*q2*(q1^2+q2^2+3*(q3^2+q4^2));
            dmzdq3 = 2*m1*q1^3+2*m1*q1*q2^2+2*m1*q1*q4^2-2*m1*q2*q3*q4+2*m2*q1^2*q2+2*m2*q1*q3*q4+2*m2*q2^3+2*m2*q2*q4^2+3*m3*q1^2*q3+3*m3*q2^2*q3+m3*q3^3+m3*q3*q4^2;
            dmzdq4 = -(-2*m1*q1^2*q2+2*m1*q1*q3*q4-2*m1*q2^3-2*m1*q2*q3^2+2*m2*q1^3+2*m2*q1*q2^2+2*m2*q1*q3^2+2*m2*q2*q3*q4-3*m3*q1^2*q4-3*m3*q2^2*q4-m3*q3^2*q4-m3*q4^3);
    
            
            H = 1/num*[daxdq1 daxdq2 daxdq3 daxdq4 num 0 0 0 0 0;
               daydq1 daydq2 daydq3 daydq4 0 num 0 0 0 0;
               dazdq1 dazdq2 dazdq3 dazdq4 0 0 num 0 0 0;
               dmxdq1 dmxdq2 dmxdq3 dmxdq4 0 0 0 num 0 0;
               dmydq1 dmydq2 dmydq3 dmydq4 0 0 0 0 num 0;
               dmzdq1 dmzdq2 dmzdq3 dmzdq4 0 0 0 0 0 num];
           
            %(Ts/2)^2*E*10*eye(3)*E'
            Q = [0.001*eye(4) zeros(4,3) zeros(4,3);
            zeros(3,4) 0.00000001*eye(3) zeros(3,3);
            zeros(3,4) zeros(3,3) eye(3)*0.00000001];
    
            P = A*P*A'+Q;       
           
            %Measurement update eq
            K = P*H'*inv(H*P*H'+R);
            x = x+K*(z-[Rot*[0 0 1]';Rot*magnometer]-x(5:10));
            P = (eye(size(K*H))-K*H)*P;
    
    
            %Converting from quaternions to euler angles(pitch,roll,yaw)
            qq = x(1:4)/norm(x(1:4));
            euler_ang=qua_to_euler(qq);
       
        %Only plot one out of ten estimates    
        if(count > 10)
            %Draw cube
            clf
            
            xx=euler_ang(1);
            yy=euler_ang(2);
            zz=euler_ang(3);
            RR=[cos(zz)*cos(yy) -sin(zz)*cos(xx)+cos(zz)*sin(yy)*sin(xx) sin(zz)*sin(xx)+cos(zz)*cos(xx)*sin(yy);
            sin(zz)*cos(yy) cos(zz)*cos(xx)+sin(xx)*sin(yy)*sin(zz) -cos(zz)*sin(xx)+sin(yy)*sin(zz)*cos(xx);
            -sin(yy) cos(yy)*sin(xx) cos(yy)*cos(xx)];
        
            %Rotation of cube
            side1=Rot90y*Rot90z*RR*side1o;
            side2=Rot90y*Rot90z*RR*side2o;
            side3=Rot90y*Rot90z*RR*side3o;
            side4=Rot90y*Rot90z*RR*side4o;
            side5=Rot90y*Rot90z*RR*side5o;
            side6=Rot90y*Rot90z*RR*side6o;
            
            hold on;
            
            %Coloring the cube
            fill3(side1(1,:),side1(2,:),side1(3,:),G1,'FaceAlpha',0.8);
            fill3(side2(1,:),side2(2,:),side2(3,:),G2,'FaceAlpha',0.8);
            fill3(side3(1,:),side3(2,:),side3(3,:),G3,'FaceAlpha',0.8);
            fill3(side4(1,:),side4(2,:),side4(3,:),G4,'FaceAlpha',0.8);
            fill3(side5(1,:),side5(2,:),side5(3,:),G5,'FaceAlpha',0.8);
            fill3(side6(1,:),side6(2,:),side6(3,:),G6,'FaceAlpha',0.8);
            
            axis([-1.5 1.5 -1.5 1.5 -1.5 1.5])
            drawnow();
            hold off;
            
            count = 1;
        end
        count = count + 1;
    end
end
fclose(s);

