%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%   Kristian L. Jenssen & Kaan H. Yabar
%   Engineering Cybernetics - April.2011 
%
%   RUNS EQUEST AND EKF ON SAVED RAW DATA FROM SPARKFUN IMU
%   "data_read2.m" is required to run the file.
%   This files draws the euler angles for estimated attitude both for EKF
%   and EQUEST
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

data_read2

%Local magnetic field for Trondheim
magnometer=[13598.5;444.7;-49854.8];
magnometer=magnometer/norm(magnometer);

x = zeros(10,1);
Ts = 0.0011;
x(1:4,1) = [0 0 0 1]';

%Covariance matrix for process and sensors
r1 = 0.05;
r2 = 0.08;
R = diag([r1,r1,r1,r2,r2,r2]);

P = 0*eye(10);

%Normalized gravity vector
a=[0 0 1];
a1=a(1);
a2=a(2);
a3=a(3);

go = true;
count=1;
q=[0,0,0,1]';
qq=[0,0,0,1]';
t=0;

euler_ang1=[0,0,0]';

%Estimate only for each 4. step
for i=1:4:2483
            
            acc = dataM(3:5,i);
            omega_real = dataM(6:8,i)-[260 257 260]';
            mag = dataM(9:11,i);
            omega_real = omega_real*0.92*(pi/180);
            mag = mag/norm(mag);
            acc = acc/norm(acc);
            
            %EKF
            z = [acc;mag];
            omega_skew=[0 -omega_real(3) omega_real(2);omega_real(3) 0 -omega_real(1);-omega_real(2) omega_real(1) 0];
            Omega = 0.5 * [omega_skew omega_real; -omega_real' 0];
    
            m1 = magnometer(1);
            m2 = magnometer(2);
            m3 = magnometer(3);
         
            A = [expm(Omega*Ts) zeros(4,3) zeros(4,3);
            zeros(3,4) eye(3) zeros(3,3);
            zeros(3,4) zeros(3,3) eye(3)];
        
            %Extended Kalman Filter Equations
            
            %Time update eq
            x = A*x;
            x(1:4)=x(1:4)/norm(x(1:4));
    
            q1 = x(1);
            q2 = x(2);
            q3 = x(3);
            q4 = x(4);
            q_13=[0 -q3 q2;q3 0 -q1;-q2 q1 0];
            E=[q4*eye(3)+q_13;-q1 -q2 -q3];
            Y=[q4*eye(3)-q_13;-q1 -q2 -q3];
            Rot=E'*Y;
            
            %Linearization of measurment matrix
            num = (q1^2+q2^2+q3^2+q4^2)^(3/2);

            daxdq1 = a1*q1^3+3*a1*q1*q2^2+3*a1*q1*q3^2+a1*q1*q4^2-2*a2*q1*q3*q4+2*a2*q2^3+2*a2*q2*q3^2+2*a2*q2*q4^2+2*a3*q1*q2*q4+2*a3*q2^2*q3+2*a3*q3^3+2*a3*q3*q4^2;
            daxdq2 = -3*a1*q1^2*q2-a1*q2^3-a1*q2*q3^2-3*a1*q2*q4^2+2*a2*q1^3+2*a2*q1*q3^2+2*a2*q1*q4^2-2*a2*q2*q3*q4-2*a3*q1^2*q4-2*a3*q1*q2*q3-2*a3*q3^2*q4-2*a3*q4^3;
            daxdq3 = -3*a1*q1^2*q3-a1*q2^2*q3-a1*q3^3-3*a1*q3*q4^2+2*a2*q1^2*q4-2*a2*q1*q2*q3+2*a2*q2^2*q4+2*a2*q4^3+2*a3*q1^3+2*a3*q1*q2^2+2*a3*q1*q4^2+2*a3*q2*q3*q4;
            daxdq4 = -(-a1*q1^2*q4-3*a1*q2^2*q4-3*a1*q3^2*q4-a1*q4^3-2*a2*q1^2*q3+2*a2*q1*q2*q4-2*a2*q2^2*q3-2*a2*q3^3+2*a3*q1^2*q2+2*a3*q1*q3*q4+2*a3*q2^3+2*a3*q2*q3^2);
            daydq1 = 2*(a1*(q1*q3*q4+q2^3+q2*(q3^2+q4^2))+a3*(q4*(q2^2+q3^2+q4^2)-q1*q2*q3))-a2*q1*(q1^2+3*q2^2+q3^2+3*q4^2);
            daydq2 = 2*a1*q1^3+2*a1*q1*q3^2+2*a1*q1*q4^2+2*a1*q2*q3*q4+3*a2*q1^2*q2+a2*q2^3+3*a2*q2*q3^2+a2*q2*q4^2+2*a3*q1^2*q3-2*a3*q1*q2*q4+2*a3*q3^3+2*a3*q3*q4^2;
            daydq3 = -2*a1*q1^2*q4-2*a1*q1*q2*q3-2*a1*q2^2*q4-2*a1*q4^3-a2*q1^2*q3-3*a2*q2^2*q3-a2*q3^3-3*a2*q3*q4^2+2*a3*q1^2*q2-2*a3*q1*q3*q4+2*a3*q2^3+2*a3*q2*q4^2;
            daydq4 = -2*a1*q1^2*q3-2*a1*q1*q2*q4-2*a1*q2^2*q3-2*a1*q3^3+3*a2*q1^2*q4+2*a2*q2^2*q4+3*a2*q3^2*q4+a2*q4^3+2*a3*q1^3+2*a3*q1*q2^2+2*a3*q1*q3^2-2*a3*q2*q3*q4;
            dazdq1 = -(2*a1*q1*q2*q4-2*a1*q2^2*q3-2*a1*q3^3-2*a1*q3*q4^2+2*a2*q1*q2*q3+2*a2*q2^2*q4+2*a2*q3^2*q4+2*a2*q4^3+a3*q1^3+a3*q1*q2^2+3*a3*q1*q3^2+3*a3*q1*q4^2);
            dazdq2 = 2*(a1*(q1^2*q4-q1*q2*q3+q3^2*q4+q4^3)+a2*(q1^2*q3+q1*q2*q4+q3^3+q3*q4^2))-a3*q2*(q1^2+q2^2+3*(q3^2+q4^2));
            dazdq3 = 2*a1*q1^3+2*a1*q1*q2^2+2*a1*q1*q4^2-2*a1*q2*q3*q4+2*a2*q1^2*q2+2*a2*q1*q3*q4+2*a2*q2^3+2*a2*q2*q4^2+3*a3*q1^2*q3+3*a3*q2^2*q3+a3*q3^3+a3*q3*q4^2;
            dazdq4 = -(-2*a1*q1^2*q2+2*a1*q1*q3*q4-2*a1*q2^3-2*a1*q2*q3^2+2*a2*q1^3+2*a2*q1*q2^2+2*a2*q1*q3^2+2*a2*q2*q3*q4-3*a3*q1^2*q4-3*a3*q2^2*q4-a3*q3^2*q4-a3*q4^3);
    
            dmxdq1 = m1*q1^3+3*m1*q1*q2^2+3*m1*q1*q3^2+m1*q1*q4^2-2*m2*q1*q3*q4+2*m2*q2^3+2*m2*q2*q3^2+2*m2*q2*q4^2+2*m3*q1*q2*q4+2*m3*q2^2*q3+2*m3*q3^3+2*m3*q3*q4^2;
            dmxdq2 = -3*m1*q1^2*q2-m1*q2^3-m1*q2*q3^2-3*m1*q2*q4^2+2*m2*q1^3+2*m2*q1*q3^2+2*m2*q1*q4^2-2*m2*q2*q3*q4-2*m3*q1^2*q4-2*m3*q1*q2*q3-2*m3*q3^2*q4-2*m3*q4^3;
            dmxdq3 = -3*m1*q1^2*q3-m1*q2^2*q3-m1*q3^3-3*m1*q3*q4^2+2*m2*q1^2*q4-2*m2*q1*q2*q3+2*m2*q2^2*q4+2*m2*q4^3+2*m3*q1^3+2*m3*q1*q2^2+2*m3*q1*q4^2+2*m3*q2*q3*q4;
            dmxdq4 = -(-m1*q1^2*q4-3*m1*q2^2*q4-3*m1*q3^2*q4-m1*q4^3-2*m2*q1^2*q3+2*m2*q1*q2*q4-2*m2*q2^2*q3-2*m2*q3^3+2*m3*q1^2*q2+2*m3*q1*q3*q4+2*m3*q2^3+2*m3*q2*q3^2);
            dmydq1 = 2*(m1*(q1*q3*q4+q2^3+q2*(q3^2+q4^2))+m3*(q4*(q2^2+q3^2+q4^2)-q1*q2*q3))-m2*q1*(q1^2+3*q2^2+q3^2+3*q4^2);
            dmydq2 = 2*m1*q1^3+2*m1*q1*q3^2+2*m1*q1*q4^2+2*m1*q2*q3*q4+3*m2*q1^2*q2+m2*q2^3+3*m2*q2*q3^2+m2*q2*q4^2+2*m3*q1^2*q3-2*m3*q1*q2*q4+2*m3*q3^3+2*m3*q3*q4^2;
            dmydq3 = -2*m1*q1^2*q4-2*m1*q1*q2*q3-2*m1*q2^2*q4-2*m1*q4^3-m2*q1^2*q3-3*m2*q2^2*q3-m2*q3^3-3*m2*q3*q4^2+2*m3*q1^2*q2-2*m3*q1*q3*q4+2*m3*q2^3+2*m3*q2*q4^2;
            dmydq4 = -2*m1*q1^2*q3-2*m1*q1*q2*q4-2*m1*q2^2*q3-2*m1*q3^3+3*m2*q1^2*q4+2*m2*q2^2*q4+3*m2*q3^2*q4+m2*q4^3+2*m3*q1^3+2*m3*q1*q2^2+2*m3*q1*q3^2-2*m3*q2*q3*q4;
            dmzdq1 = -(2*m1*q1*q2*q4-2*m1*q2^2*q3-2*m1*q3^3-2*m1*q3*q4^2+2*m2*q1*q2*q3+2*m2*q2^2*q4+2*m2*q3^2*q4+2*m2*q4^3+m3*q1^3+m3*q1*q2^2+3*m3*q1*q3^2+3*m3*q1*q4^2);
            dmzdq2 = 2*(m1*(q1^2*q4-q1*q2*q3+q3^2*q4+q4^3)+m2*(q1^2*q3+q1*q2*q4+q3^3+q3*q4^2))-m3*q2*(q1^2+q2^2+3*(q3^2+q4^2));
            dmzdq3 = 2*m1*q1^3+2*m1*q1*q2^2+2*m1*q1*q4^2-2*m1*q2*q3*q4+2*m2*q1^2*q2+2*m2*q1*q3*q4+2*m2*q2^3+2*m2*q2*q4^2+3*m3*q1^2*q3+3*m3*q2^2*q3+m3*q3^3+m3*q3*q4^2;
            dmzdq4 = -(-2*m1*q1^2*q2+2*m1*q1*q3*q4-2*m1*q2^3-2*m1*q2*q3^2+2*m2*q1^3+2*m2*q1*q2^2+2*m2*q1*q3^2+2*m2*q2*q3*q4-3*m3*q1^2*q4-3*m3*q2^2*q4-m3*q3^2*q4-m3*q4^3);
    
    
            H = 1/num*[daxdq1 daxdq2 daxdq3 daxdq4 num 0 0 0 0 0;
               daydq1 daydq2 daydq3 daydq4 0 num 0 0 0 0;
               dazdq1 dazdq2 dazdq3 dazdq4 0 0 num 0 0 0;
               dmxdq1 dmxdq2 dmxdq3 dmxdq4 0 0 0 num 0 0;
               dmydq1 dmydq2 dmydq3 dmydq4 0 0 0 0 num 0;
               dmzdq1 dmzdq2 dmzdq3 dmzdq4 0 0 0 0 0 num];
           
            %(Ts/2)^2*E*10*eye(3)*E'
            Q = [0.001*eye(4) zeros(4,3) zeros(4,3);
            zeros(3,4) 0.00000001*eye(3) zeros(3,3);
            zeros(3,4) zeros(3,3) eye(3)*0.00000001];
    
            P = A*P*A'+Q;       
           
            %Measurement update eq
            K = P*H'*inv(H*P*H'+R);
            x = x+K*(z-[Rot*[0 0 1]';Rot*magnometer]-x(5:10));
            P = (eye(size(K*H))-K*H)*P;
    
    
            %converting from quaternions to euler angles(pitch,roll,yaw)
            qq = x(1:4)/norm(x(1:4));
            euler_ang1=qua_to_euler(qq);
            
            %EQUEST
            q_13=[0 -q(3) q(2);q(3) 0 -q(1);-q(2) q(1) 0];
            E=[q(4)*eye(3)+q_13;-q(1) -q(2) -q(3)];
            delta_q=0.5*E*omega_real;
            q=q+delta_q*0.01;
            q=q/norm(q);
    
            B=(1/0.001)*acc*[0 0 1] + (1/0.001)*mag*magnometer';  %(1/0.05)*(mag/norm(mag))*magnometer
            S=B+B';
            Z=[B(2,3)-B(3,2);B(3,1)-B(1,3);B(1,2)-B(2,1)];
            o=trace(B);
            V=[S-o*eye(3) Z;Z' o];
            KK=-V*20+eye(4)*100;
            %Using eigenvalues to find optimal q
            xi=-eye(4)*q*100;
            optimal_lam = optimal_lambda(KK,xi);
            y = -inv(KK+optimal_lam*eye(4))*xi;

    
    
            %Converting from quaternions to euler angles(pitch,roll,yaw)
            q=y(1:4);
            q = q/norm(q);
            euler_ang2 = qua_to_euler(q);
            
            
            %Plot euler angles
            subplot(3,1,1)
            hold on;
            plot(t,euler_ang2(1)*180/pi,'color','red');
            plot(t,euler_ang1(1)*180/pi,'color','blue');
            axis([0 inf -180 180]); 
            legend('EQUEST','EKF')
            xlabel('Time [s]');
            ylabel('Roll [Deg]');
            
            subplot(3,1,2)
            hold on;
            plot(t,euler_ang2(2)*180/pi,'color','red');
            plot(t,euler_ang1(2)*180/pi,'color','blue');
            axis([0 inf -180 180]); 
            xlabel('Time [s]');
            ylabel('Pitch [Deg]');
            
            subplot(3,1,3)
            hold on;
            plot(t,euler_ang2(3)*180/pi,'LineWidth',1,'color','red');
            plot(t,euler_ang1(3)*180/pi,'LineWidth',1,'color','blue');
            axis([0 inf -180 180]); 
            xlabel('Time [s]');
            ylabel('Yaw [Deg]');
            
            t=t+0.04;
            
end


